'use strict';

/* global MathJax */
var React = require('react');
var createReactClass = require('create-react-class');
var PropTypes = require('prop-types');
var loadScript = require('load-script');

var DEFAULT_SCRIPT = 'https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.1/MathJax.js?config=TeX-MML-AM_CHTML';

var DEFAULT_OPTIONS = {
    tex2jax: {
        inlineMath: []
    },
    showMathMenu: false,
    showMathMenuMSIE: false
};

/**
 * Context for loading mathjax
 * @type {[type]}
 */
var MathJaxContext = createReactClass({
    displayName: 'MathJaxContext',
    getDefaultProps: function getDefaultProps() {
        return {
            script: DEFAULT_SCRIPT,
            options: DEFAULT_OPTIONS
        };
    },
    getInitialState: function getInitialState() {
        return {
            loaded: false
        };
    },
    getChildContext: function getChildContext() {
        return {
            MathJax: typeof MathJax == 'undefined' ? undefined : MathJax
        };
    },
    componentDidMount: function componentDidMount() {
        var script = this.props.script;


        if (!script) {
            return this.onLoad();
        }

        loadScript(script, this.onLoad);
    },
    onLoad: function onLoad(err) {
        var _this = this;

        var options = this.props.options;

        MathJax.Hub.Config(options);

        MathJax.Hub.Register.StartupHook("End", function () {
            if (_this.props.didFinishTypeset) {
                _this.props.didFinishTypeset();
            }
        });

        this.setState({
            loaded: true
        });
    },
    render: function render() {
        var children = this.props.children;

        return React.Children.only(children);
    }
});

MathJaxContext.propTypes = {
    children: PropTypes.node.isRequired,
    didFinishTypeset: PropTypes.func,
    script: PropTypes.oneOfType([PropTypes.string, PropTypes.oneOf([false])]),
    options: PropTypes.object
};

MathJaxContext.childContextTypes = {
    MathJax: PropTypes.object
};

module.exports = MathJaxContext;